@@ -115,11 +115,11 @@ install_auto_save_timeout (GeditTab *tab)
 	g_return_if_fail (tab->priv->state != GEDIT_TAB_STATE_SAVING_ERROR);
 	g_return_if_fail (tab->priv->state != GEDIT_TAB_STATE_SAVING_ERROR);
 	g_return_if_fail (tab->priv->state != GEDIT_TAB_STATE_REVERTING_ERROR);
-	
+
 	/* Add a new timeout */
-	timeout = g_timeout_add (tab->priv->auto_save_interval * 1000 * 60,
-				 (GSourceFunc) gedit_tab_auto_save,
-				 tab);
+	timeout = g_timeout_add_seconds (tab->priv->auto_save_interval * 60,
+					 (GSourceFunc) gedit_tab_auto_save,
+					 tab);
 
 	tab->priv->auto_save_timeout = timeout;
 }
@@ -128,7 +128,7 @@ static gboolean
 install_auto_save_timeout_if_needed (GeditTab *tab)
 {
 	GeditDocument *doc;
-	
+
 	gedit_debug (DEBUG_TAB);
 	
 	g_return_val_if_fail (tab->priv->auto_save_timeout <= 0, FALSE);
@@ -289,6 +289,14 @@ gedit_tab_class_init (GeditTabClass *klass)
 	g_type_class_add_private (object_class, sizeof (GeditTabPrivate));
 }
 
+/**
+ * gedit_tab_get_state:
+ * @tab: a #GeditTab
+ *
+ * Gets the #GeditTabState of @tab.
+ *
+ * Returns: the #GeditTabState of @tab
+ */
 GeditTabState
 gedit_tab_get_state (GeditTab *tab)
 {
@@ -798,9 +806,9 @@ message_area_set_progress (GeditTab *tab,
 {
 	if (tab->priv->message_area == NULL)
 		return;
-	
-	gedit_debug_message (DEBUG_TAB, "%Ld/%Ld", size, total_size);
-	
+
+	gedit_debug_message (DEBUG_TAB, "%" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT, size, total_size);
+
 	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
 	
 	if (total_size == 0)
@@ -836,8 +844,8 @@ document_loading (GeditDocument *document,
 	g_return_if_fail ((tab->priv->state == GEDIT_TAB_STATE_LOADING) ||
 		 	  (tab->priv->state == GEDIT_TAB_STATE_REVERTING));
 
-	gedit_debug_message (DEBUG_TAB, "%Ld/%Ld", size, total_size);
-	
+	gedit_debug_message (DEBUG_TAB, "%" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT, size, total_size);
+
 	if (tab->priv->timer == NULL)
 	{
 		g_return_if_fail (tab->priv->times_called == 0);
@@ -1047,8 +1055,9 @@ document_loaded (GeditDocument *document,
 			    		g_object_unref (loc);
 			    		break;
 			    	}
-
-				g_object_unref (loc);
+			    	
+			    	if (loc != NULL)
+					g_object_unref (loc);
 			}
 		}
 
@@ -1079,8 +1088,9 @@ document_saving (GeditDocument    *document,
 
 	g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_SAVING);
 
-	gedit_debug_message (DEBUG_TAB, "%Ld/%Ld", size, total_size);
-	
+	gedit_debug_message (DEBUG_TAB, "%" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT, size, total_size);
+
+
 	if (tab->priv->timer == NULL)
 	{
 		g_return_if_fail (tab->priv->times_called == 0);
@@ -1611,13 +1621,28 @@ _gedit_tab_new_from_uri (const gchar         *uri,
 	return GTK_WIDGET (tab);
 }		
 
+/**
+ * gedit_tab_get_view:
+ * @tab: a #GeditTab
+ *
+ * Gets the #GeditView inside @tab.
+ *
+ * Returns: the #GeditView inside @tab
+ */
 GeditView *
 gedit_tab_get_view (GeditTab *tab)
 {
 	return GEDIT_VIEW (tab->priv->view);
 }
 
-/* This is only an helper function */
+/**
+ * gedit_tab_get_document:
+ * @tab: a #GeditTab
+ *
+ * Gets the #GeditDocument associated to @tab.
+ *
+ * Returns: the #GeditDocument associated to @tab
+ */
 GeditDocument *
 gedit_tab_get_document (GeditTab *tab)
 {
@@ -1693,9 +1718,10 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 
 	switch (tab->priv->state)
 	{
+		gchar *content_type;
 		gchar *mime_type;
-		gchar *mime_description;
-		gchar *mime_full_description; 
+		gchar *content_description;
+		gchar *content_full_description; 
 		gchar *encoding;
 		const GeditEncoding *enc;
 
@@ -1714,16 +1740,17 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 						ruri_markup);
 			break;			
 		default:
+			content_type = gedit_document_get_content_type (doc);
 			mime_type = gedit_document_get_mime_type (doc);
-			mime_description = g_content_type_get_description (mime_type);
+			content_description = g_content_type_get_description (content_type);
 
-			if (mime_description == NULL)
-				mime_full_description = g_strdup (mime_type);
+			if (content_description == NULL)
+				content_full_description = g_strdup (mime_type);
 			else
-				mime_full_description = g_strdup_printf ("%s (%s)", 
-						mime_description, mime_type);
+				content_full_description = g_strdup_printf ("%s (%s)", 
+						content_description, mime_type);
 
-			g_free (mime_description);
+			g_free (content_description);
 			g_free (mime_type);
 
 			enc = gedit_document_get_encoding (doc);
@@ -1737,15 +1764,15 @@ _gedit_tab_get_tooltips	(GeditTab *tab)
 						        "<b>%s</b> %s\n"
 						        "<b>%s</b> %s",
 						        _("Name:"), ruri,
-						        _("MIME Type:"), mime_full_description,
+						        _("MIME Type:"), content_full_description,
 						        _("Encoding:"), encoding);
 
 			g_free (encoding);
-			g_free (mime_full_description);
+			g_free (content_full_description);
 
 			break;
 	}
-	
+
 	g_free (ruri);	
 	g_free (ruri_markup);
 	
@@ -1935,6 +1962,14 @@ _gedit_tab_get_icon (GeditTab *tab)
 	return pixbuf;
 }
 
+/**
+ * gedit_tab_get_from_document:
+ * @doc: a #GeditDocument
+ *
+ * Gets the #GeditTab associated with @doc.
+ *
+ * Returns: the #GeditTab associated with @doc
+ */
 GeditTab *
 gedit_tab_get_from_document (GeditDocument *doc)
 {
@@ -2088,18 +2123,18 @@ gedit_tab_auto_save (GeditTab *tab)
 	if ((tab->priv->state != GEDIT_TAB_STATE_NORMAL) &&
 	    (tab->priv->state != GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW))
 	{
-		/* Retry after 15 seconds */
+		/* Retry after 30 seconds */
 		guint timeout;
-		
-		gedit_debug_message (DEBUG_TAB, "Retry after 15 seconds");
-	
+
+		gedit_debug_message (DEBUG_TAB, "Retry after 30 seconds");
+
 		/* Add a new timeout */
-		timeout = g_timeout_add (30 * 1000,
-					(GSourceFunc) gedit_tab_auto_save,
-					tab);
+		timeout = g_timeout_add_seconds (30,
+						 (GSourceFunc) gedit_tab_auto_save,
+						 tab);
 
 		tab->priv->auto_save_timeout = timeout;
-	    
+
 	    	/* Returns FALSE so the old timeout is "destroyed" */
 		return FALSE;
 	}
@@ -2619,11 +2654,11 @@ _gedit_tab_can_close (GeditTab *tab)
 
 /**
  * gedit_tab_get_auto_save_enabled:
- * @tab: 
+ * @tab: a #GeditTab
  * 
  * Gets the current state for the autosave feature
  * 
- * Return value: TRUE if the autosave is enabled, else FALSE
+ * Return value: %TRUE if the autosave is enabled, else %FALSE
  **/
 gboolean
 gedit_tab_get_auto_save_enabled	(GeditTab *tab)
@@ -2638,7 +2673,7 @@ gedit_tab_get_auto_save_enabled	(GeditTab *tab)
 /**
  * gedit_tab_set_auto_save_enabled:
  * @tab: a #GeditTab
- * @enable: enable (TRUE) or disable (FALSE) auto save
+ * @enable: enable (%TRUE) or disable (%FALSE) auto save
  * 
  * Enables or disables the autosave feature. It does not install an
  * autosave timeout if the document is new or is read-only
@@ -2699,7 +2734,7 @@ gedit_tab_set_auto_save_enabled	(GeditTab *tab,
 
 /**
  * gedit_tab_get_auto_save_interval:
- * @tab: 
+ * @tab: a #GeditTab
  * 
  * Gets the current interval for the autosaves
  * 
