@@ -24,7 +24,7 @@ import gtksourceview2 as gsv
 import os.path
 from library import *
 from functions import *
-import md5
+import hashlib
 
 class Manager(object):
     UI_FILE = os.path.join(os.path.dirname(__file__), "tools.ui")
@@ -119,7 +119,7 @@ class Manager(object):
             return None, None
 
     def compute_hash(self, string):
-        return md5.md5(string).hexdigest()
+        return hashlib.md5(string).hexdigest()
 
     def save_current_tool(self):
         if self.current_node is None:
@@ -166,9 +166,9 @@ class Manager(object):
         script = default(''.join(node.get_script()), '')
         buf.set_text(script)
         self.script_hash = self.compute_hash(script)
-        mimetype = gio.content_type_guess(data=script)
+        contenttype = gio.content_type_guess(data=script)
         lmanager = gedit.get_language_manager()
-        language = gedit.language_manager_get_language_from_mime_type(lmanager, mimetype)
+        language = lmanager.guess_language(content_type=contenttype)
 
         if language is not None:
             buf.set_language(language)
